/*
  exampleSetup_MEGA
  Modifies the setup of HerkuleX servos thanks to the HerkuleXLib
  (see www.benoitpuel.com/blog/telechargements/) using an Arduino MEGA 
  board.

  The serial terminal is used to printout messages.

  This example code is in the public domain.

  modified 9 February 2016
  by Benoit Puel
 */

#include <HkxSetup.h>

void setup() {
  delay(5000); // wait 5 seconds to open the serial monitor
  HkxPrint printoutLib = HkxPrint(Serial, 9600);  // Printout errors on the serial monitor
  HkxPrint printoutCode = HkxPrint(printoutLib, true, true, true);  // Printout errors on the serial monitor
  HkxCommunication communication = HkxCommunication(HKX_115200, Serial1, printoutLib);  // Communication with the servo on Serial1
  HkxSetup setupServos(communication, printoutLib);  // setup the servos

  // Variables
  HkxStatus statusED;
  bool IDs[254];
  uint16_t voltage;
  uint16_t temperature;
  String message;

  // Modify the baudrate
  setupServos.setAllBaudRate(HKX_666666);

  // Modify the Load controls
  setupServos.setAllLoadControl(0, 200, 50, 0, 0, 550);

  // Modify the PWM and position limits
  setupServos.setAllPWMPositionLimits(600, -1667, 1665);

  // Modify the control system
  setupServos.setAllControlSystem(500, 10000, 0, 20000, 0);

  // Modify the position criteria
  setupServos.setAllInPositionCriteria(3, 3);

  // Scan the IDs 
  printoutCode.infoPrint("Start the ID scan (can take some time)");
  setupServos.scanIDs(IDs);
  printoutCode.infoPrint("ID scan performed");

  for(int ID=0 ; ID < 254 ; ID++){
    if(IDs[ID]){
      message = "ID = ";
      message += ID;
      printoutCode.infoPrint(message);
      
      // Get the behaviour
      setupServos.getBehaviour(ID, &voltage, &temperature, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE, HKX_NO_VALUE);
      message = "voltage = ";
      message += (float)voltage/1000;
      message += " V \t temperature = ";
      message += (float)temperature/100;
      message += " °C";
      printoutCode.infoPrint(message);
      
      // Get the status
      setupServos.getStatus(ID, statusED);
      if(statusED.isError(HKX_STAT_ALL)){
        printoutCode.errorPrint("The servo has an error");
        if(statusED.isError(HKX_STAT_VOLTAGE)){
          printoutCode.errorPrint("Exceed Input Voltage limit");
        } if(statusED.isError(HKX_STAT_POSITION)){
          printoutCode.errorPrint("Exceed allowed POT limit");
        } if(statusED.isError(HKX_STAT_TEMPERATURE)){
          printoutCode.errorPrint("Exceed Temperature limit");
        } if(statusED.isError(HKX_STAT_PACKET)){
          printoutCode.errorPrint("Invalid Packet");
        } if(statusED.isError(HKX_STAT_OVERLOAD)){
          printoutCode.errorPrint("Overload detected");
        } if(statusED.isError(HKX_STAT_DRIVER)){
          printoutCode.errorPrint("Driver fault detected");
        } if(statusED.isError(HKX_STAT_ROM_DISTORTED)){
          printoutCode.errorPrint("EEP REG distorted");
        }
      }
    }
  }
  setupServos.clearAllStatus ();
}

void loop() {
  // put your main code here, to run repeatedly:

}
